# -*- coding: utf-8 -*-
##
## -----------------------------------------------------------------
##    This file is part of WAPT Software Deployment
##    Copyright (C) 2012 - 2024  Tranquil IT https://www.tranquil.it
##    All Rights Reserved.
##
##    WAPT helps systems administrators to efficiently deploy
##    setup, update and configure applications.
## ------------------------------------------------------------------
##

import os
import sys
import dialog
sys.path.append('/opt/wapt')
from optparse import OptionParser
from setuphelpers import wget,wgets,type_debian,type_redhat
from requests import HTTPError

if "__file__" in locals():
    sys.path.insert(0, os.path.realpath(os.path.join( os.path.dirname(os.path.realpath(__file__)),'..','..')))

from setuphelpers import *
import waptserver.config
import grp
import pwd as good_pwd

postconf = dialog.Dialog(dialog="dialog")

if type_debian():
    NGINX_ACCOUNT = 'www-data'
    NGINX_GID = grp.getgrnam(NGINX_ACCOUNT).gr_gid
elif type_redhat():
    NGINX_ACCOUNT = 'nginx'
    NGINX_GID = grp.getgrnam(NGINX_ACCOUNT).gr_gid

WAPT_UID = good_pwd.getpwnam('wapt').pw_uid

def get_waptagent(tmpdir='/tmp',config='/opt/wapt/conf/waptserver.ini',quiet=False,allagents=False, url=None):

    with open('/opt/wapt/version-full','r') as f:
        data = f.read()

    server_config = waptserver.config.load_config(config)
    proxiesconf = server_config['http_proxy']

    proxies={"https":proxiesconf}
    if not url:
        url = 'https://wapt.tranquil.it/wapt/releases/wapt-' + data + '/wapt_agent_list.json'
    try:
        datajson = wgets(url, as_json=True,proxies=proxies)
    except HTTPError as e :
        if not e.response.status_code == 404:
            raise
        url = 'https://wapt.tranquil.it/wapt/nightly/wapt-' + data + '/wapt_agent_list.json'
        datajson = wgets(url, as_json=True,proxies=proxies)

    waptagent_folder =  os.path.join(server_config['wapt_folder'], 'waptagent')
    datajson = wgets(url, as_json=True,proxies=proxies)

    if allagents:
        reply = [u['url'].split('/')[-1] for u in datajson]

    elif quiet :
        reply = []
        for u in datajson :
            fname = u['url'].split('/')[-1]
            if os.path.isfile(os.path.join(waptagent_folder,u['filename_without_version'])):
                reply.append(fname)
    else:
        (code, reply) = postconf.checklist(text='Select the wapt agents you need from the list.',height=20,width=100,choices=[(u['url'].split('/')[-1],"",os.path.isfile(os.path.join(waptagent_folder,u['filename_without_version']))) for u in datajson])
        if code != postconf.DIALOG_OK:
            exit(1)


    for u in datajson:
        urldl = u['url']
        finalfilename = os.path.join(waptagent_folder,u['filename_without_version'])
        fname = urldl.split('/')[-1]
        if not fname in reply:
            continue

        print('Download %s' % fname)
        wget(urldl,finalfilename, proxies=proxies, sha256=u['file_sha256'])

        open(finalfilename+'.sha256','w').write(u['file_sha256'])
        for f in (finalfilename,finalfilename+'.sha256'):
            os.chown(f, WAPT_UID, NGINX_GID)
            os.chmod(f, 0o750)


def main():

    usage = """%prog [--config filename]"""

    parser = OptionParser(usage=usage, version=__version__)
    parser.add_option(
        '-c',
        '--config',
        dest='configfile',
        default='/opt/wapt/conf/waptserver.ini',
        help='Config file full path (default: %default)')

    parser.add_option(
        '-q',
        '--quiet',
        dest='quiet',
        default=False,
        action="store_true",
        help='Run quiet download agent')

    parser.add_option(
        '-a',
        '--allagents',
        dest='allagents',
        default=False,
        action="store_true",
        help='download all agent')

    parser.add_option(
        '-u',
        '--url',
        dest='url',
        default=None,
        help='wapt_agent_list.json url')




    (options, args) = parser.parse_args()
    get_waptagent(config=options.configfile,quiet=options.quiet,allagents=options.allagents, url=options.url)

if __name__ == "__main__":
    if not type_debian() and not type_redhat():
        print("Unsupported distribution")
        sys.exit(1)
    main()
