import os
import sys
import getpass
sys.path.append('/opt/wapt')
from optparse import OptionParser

if "__file__" in locals():
    sys.path.insert(0, os.path.realpath(os.path.join( os.path.dirname(os.path.realpath(__file__)),'..','..')))

import logging
logging.basicConfig()
logging.getLogger().setLevel(logging.DEBUG)

import auth_module_ad
import waptserver.config
import re


def test_ldap(config='/opt/wapt/conf/waptserver.ini'):

    server_config = waptserver.config.load_config(config)

    assert isinstance(server_config.get('ad_domain_name'), str) or (
        isinstance(server_config.get('ldap_auth_server'), str) and isinstance(server_config.get('ldap_auth_base_dn'), str)
    ), f"You need to configure 'ad_domain_name' or both 'ldap_auth_server' and 'ldap_auth_base_dn' in your '{waptserver.config.DEFAULT_CONFIG_FILE}'"

    dc_base_dn = server_config.get('ldap_auth_base_dn')
    if dc_base_dn:
       realm = dc_base_dn.lower().split('dc=', 1)[-1].replace('dc=', '.').replace(',', '').upper()

       with open('/etc/krb5.conf','r') as f:
            krb5conf = f.read()
       if not realm in krb5conf:
          print('ERROR : please configure your /etc/krb5.conf\n')
          print(r"""[libdefaults]
default_realm = %s
dns_lookup_kdc = true
dns_lookup_realm = true""" % realm)
          sys.exit(1)

    if server_config.get('ldap_auth_server',''):
       pattern = r'^((25[0-5]|2[0-4][0-9]|[0-1]?[0-9]{1,2})\.){3}(25[0-5]|2[0-4][0-9]|[0-1]?[0-9]{1,2})$'
       if re.match(pattern, server_config.get('ldap_auth_server','')) is not None :
          print('ldap_auth_server %s is an ip, please configure dns address' % server_config.get('ldap_auth_server',''))
          sys.exit(1)


    if server_config['ldap_account_service_login']:
        print('----------------------------------------------------------------')
        print('Test SSO SELFSERVICE LDAP with ldap_account_service_login')
        print('----------------------------------------------------------------')
        result = auth_module_ad.check_credentials_ad(server_config, '', '', [], [],use_service_account=True)
        if not result['success']:
             print(result)
             sys.exit(1)
        username_test = input('Username : ')
        group_test = input('Group test member : ')
        result = auth_module_ad.check_credentials_ad(server_config, username_test,'', [], [group_test],use_service_account=True)
        if not group_test in result['groups']:
             print('----------------------------------------------------------------')
             print('[FAIL] username %s not in group %s' %(username_test,group_test))
             print('----------------------------------------------------------------')
             print(result)
             sys.exit(1)

        print('----------------------------------------------------------------')
        print('[OK] Test SSO SELFSERVICE LDAP with ldap_account_service_login')
    else:
        print('----------------------------------------------------------------')
        print('WARNING: No ldap_account_service_login not found in %s' % config)
        print('SSO self service authentication will not work')
        print('----------------------------------------------------------------')


    print('----------------------------------------------------------------')
    print('Test ldap with direct Login ')
    print('----------------------------------------------------------------')
    username_ldap = input('Username ldap: ')
    password_ldap  = getpass.getpass(prompt='Password ldap: ', stream=None)

    result = auth_module_ad.check_credentials_ad(server_config, username_ldap, password_ldap, [], [])
    if not result['success']:
         print('----------------------------------------------------------------')
         print(result)
         sys.exit(1)

    group_test = input('Group test member : ')
    result = auth_module_ad.check_credentials_ad(server_config, username_ldap, password_ldap, [], [group_test])
    if not group_test in result['groups']:
         print('----------------------------------------------------------------')
         print('[FAIL] username %s not in group %s' %(username_ldap,group_test))
         print('----------------------------------------------------------------')
         print(result)
         sys.exit(1)

    print('--------')
    print('ALL GOOD')
    print('--------')

def main():

    usage = """%prog [--config filename]"""

    parser = OptionParser(usage=usage)
    parser.add_option(
        '-c',
        '--config',
        dest='configfile',
        default='/opt/wapt/conf/waptserver.ini',
        help='Config file full path (default: %default)')

    (options, args) = parser.parse_args()
    test_ldap(config=options.configfile)

if __name__ == "__main__":
    main()
